#pragma once

#include <stdio.h>
#include <string>
using namespace std;

#ifndef DCM_LONG_STRING_SIZE
#define	DCM_LONG_STRING_SIZE	64
#endif
typedef unsigned short PIXEL_FORMAT;
typedef unsigned __int64	TUID;
typedef void (cdecl* TOnScannerStatusEvent) (int scannerStatus);
typedef void (cdecl* TOnScannerNotifyEvent) (int scannerNotify, int lParam);
typedef void (cdecl* TOnRFIDStatusEvent) (int rfidStatus);
typedef void (cdecl* TOnRFIDNotifyEvent) (int rfidnotify);
typedef void (cdecl* TOnScannerListEvent) (int scannerCount);
typedef void (cdecl* TOnImageListEvent) (int imageCount);

#define  CSO_PAINT_CORNER  0x7100

typedef union TIPAddress
{
   unsigned long quad;
   unsigned char bytes[4];
public:
   void fromString(const char* ip)
   {
      int v0, v1, v2, v3;
      sscanf_s(ip, "%u.%u.%u.%u", &v3, &v2, &v1, &v0);
      quad =
         ((v0 & 0xFF) << 24) |
         ((v1 & 0xFF) << 16) |
         ((v2 & 0xFF) << 8) |
         (v3 & 0xFF);
   }
   string toString()
   {
      char temp[32];
      sprintf_s(temp, "%u.%u.%u.%u", bytes[0], bytes[1], bytes[2], bytes[3]);
      return temp;

   }
} TIPAddress;

typedef enum
{
   esnNone,
   esnScanStarted,
   esnProgress,
   esnPreprocessed,
   esnChangeFormat,
   esnScannerError,
   esnCalPassed,
   esnCalFailed,
   esnImageList,
   esnChangePID
} EScannerNotify;

typedef enum
{
   eisNone,
   eisIP0,
   eisIP1,
   eisIP2,
   eisIP3,
   eisIP4C,
   eisIP4,
} EIPSize;

typedef enum 
{
   ssNone,
   ssReady,
   ssSleep,
   ssScanning,
   ssErasing,
   ssTransferring,
   ssDoorJam,
   ssFlushing,
   ssNoResponse
} EUnitStatus;

typedef enum 
{
   elrOk,
   elrNone,
   elrCantLock,
   elrCantUse,
}ELockResult;

typedef enum
{
   esrSD,
   esrHD,
} EScanResolution;

typedef enum
{
   estNone,
   estDisconnected,
   estWaiting,
   estSynchronizing,
   estConnected,
   estSleeping,
   estTransferring,
   estScanning,
   estErasing,
} EScannerStatus;

typedef enum
{
   ersNone,
   ersDisconnected,
   ersConnected,
} ERFIDStatus;

typedef enum
{
   eseOk,
   eseDoorJam,
   eseDuplicateSDK,
} EScannerError;

typedef enum
{
   ectNone,
   ect1to1,
   ectNtoN,
   ectUSB,
} EConnectionType;

typedef enum
{
   ernNone,
   ernFromFireID,
   ernFromImage,
   ernFromScanner,
   ernFromOtherSDK,
} ERFIDNotify;

typedef enum
{
   eltNoConn,
   eltFireID,
   eltManual
} ELANConnectType;

union TScannerItem
{
   int params[14];
   struct {
      TIPAddress        scannerIP;
      TIPAddress			hostIP;
      EUnitStatus       unitStatus;
      char					scannerName[20];
      char					hostName[20];
      int					imageCount;      
   };
   bool operator==(TScannerItem& rValue) 
   { 
      return (scannerIP.quad == rValue.scannerIP.quad); 
   }

   TScannerItem&	operator=(TScannerItem& rValue) 
   {
      for (int i = 0; i < 14; i++)
         params[i] = rValue.params[i]; 
      return (*this);
   }
};

union TImageItem
{
   char filler[512];
   struct
   {
      char              header[4];
      TIPAddress        owner;
      long  	         index;
      unsigned short 	width;
      unsigned short 	height;
      EScanResolution   resolution;
      EIPSize   		   ipSize;
      TUID              uid;
      double  		      timeSaved;
      double      	   timeServed;
      char			      pid[DCM_LONG_STRING_SIZE];
      PIXEL_FORMAT* 	   thumbs;
   };
   bool operator==(TImageItem& rValue) { return (owner.quad == rValue.owner.quad && index == rValue.index); }
   TImageItem& operator=(TImageItem& rValue) { for (int i = 0; i < 512; i++)filler[i] = rValue.filler[i]; return (*this); }
};

typedef struct TCallBackSet
{
   TOnScannerStatusEvent	   scannerStatusEvent;
   TOnScannerNotifyEvent	   scannerNotifyEvent;
   TOnRFIDStatusEvent		   rfidStatusEvent;
   TOnRFIDNotifyEvent		   rfidNotifyEvent;
   TOnScannerListEvent        scannerListEvent;
   TOnImageListEvent          imageListEvent;   
} TCallBackSet;

typedef enum
{
  
   NERR_SUCCESS,
   NERR_FAIL_TO_CREATE_WINDOW,
   NERR_FAIL_TO_LOAD_CAL_FILE,
   NERR_FAIL_TO_START_WSA,
   NERR_FAIL_TO_START_UDP_LISTEN,
   NERR_NIC_NOT_EXIST,
   NERR_CONNECTED_ALREADY,
   NERR_FAIL_TO_CONNECT,
   NERR_REQUEST_TIMEOUT,
   NERR_NO_CONNNECTED_SCANNER,
   NERR_NOT_READY_STATUS,
   NERR_FAIL_TO_SEND_PACKET,
   NERR_INVALID_SCANNER_STATUS,
   NERR_CONNECTED_BY_FIREID,
   NERR_NOT_LAN_TYPE,
   NERR_PROC_PRE_REQ,
   NERR_SDK_OPENED_ALREADY,
   NERR_NEED_NULL_CHAR,
   NERR_SDK_NOT_OPEN,
   NERR_OUT_OF_INDEX,
   NERR_SCANNER_NOT_EXIST,
   NERR_NO_SUPPORT_1to1,
   NERR_INVALID_ARGUMENT,
   NERR_INVALID_SIZE,
   NERR_INVALID_OPTION
} ErrorCodes;

#define DLLEXPORT __declspec(dllexport)

#ifdef __cplusplus
extern "C"
{
#endif
   DLLEXPORT   BOOL              OpenScannerSDK(HWND hwnd, TCallBackSet* callbackSet, wchar_t* configPath);
   DLLEXPORT   void              CloseScannerSDK();
   DLLEXPORT   BOOL              OpenScanner();
   DLLEXPORT   BOOL              OpenScanner2(EConnectionType contype);
   DLLEXPORT   void              CloseScanner();
   DLLEXPORT   TScannerItem*  	RequestScannerList(int* count);
   DLLEXPORT   TImageItem*       RequestImageList(int* count); 
   DLLEXPORT   BOOL		  		   RequestImageFromList(TIPAddress ipAddr, int index);
   DLLEXPORT   int               GetImageWidth();
   DLLEXPORT   int               GetImageHeight();
   DLLEXPORT   PIXEL_FORMAT*     GetImageBuffer();
   DLLEXPORT   EIPSize           GetIPSize();
   DLLEXPORT   EScanResolution   GetImageResolution();
   DLLEXPORT   BOOL              ShowCalibrationDialog(BOOL visible);
   DLLEXPORT   void              ShowScannerControlDialog(BOOL visible);
   DLLEXPORT   int               GetProgress();
   DLLEXPORT   EScannerStatus    GetScannerStatus();
   DLLEXPORT 	ERFIDStatus       GetRFIDReaderStatus();
   DLLEXPORT 	TUID				   GetUID(ERFIDNotify device);
   DLLEXPORT 	EConnectionType	GetScannerConnectionType();
   DLLEXPORT	BOOL				   ClearImages();
   DLLEXPORT	char*				   GetPatientID(ERFIDNotify device);
   DLLEXPORT	BOOL				   SetPatientID(const char* pid, int size = DCM_LONG_STRING_SIZE);
   DLLEXPORT	long				   GetFirmwareVersion();
   DLLEXPORT	BOOL				   SleepScanner(BOOL dosleep);
   DLLEXPORT   BOOL              Connect(const char * ip);
   DLLEXPORT   const char *      Disconnect();
   DLLEXPORT   ErrorCodes        GetErrorCode();
   DLLEXPORT   const char *      GetErrorMessage(ErrorCodes errCode);
   DLLEXPORT   ELANConnectType   GetLANConnType();
   DLLEXPORT	int					GetScannerPN(char * buffer, int buffer_size);
   DLLEXPORT   bool              SetOption(int opt, int flag, void * optval, int valsize);

   // Deprecated
   DLLEXPORT   ELockResult       LockScanner();
   DLLEXPORT   void              UnlockScanner();
   DLLEXPORT   void              FreeImageBuffer();
   DLLEXPORT   void              SetRFIDListening(BOOL listening);
   DLLEXPORT	BOOL				   GetRFIDListening();
   DLLEXPORT 	EScannerError		GetScannerError();
   DLLEXPORT	BOOL				   DoRun();
   DLLEXPORT 	void				   SetResolution(EScanResolution res);
   DLLEXPORT	void				   SetAutoContrast(BOOL autoContrast);
   DLLEXPORT	void				   FreeUID(TUID uid);
#ifdef __cplusplus
}
#endif

typedef 	BOOL                    (* OPENSCANNERSDK)(HWND hwnd, TCallBackSet* callbackSet, wchar_t* configPath);
typedef 	void              	   (*	CLOSESCANNERSDK)();
typedef 	BOOL                    (*	OPENSCANNER)();
typedef 	BOOL                    (* OPENSCANNER2)(EConnectionType contype);
typedef 	void              	   (*	CLOSESCANNER)();
typedef 	TScannerItem*  		   (*	REQUESTSCANNERLIST)(int* count);
typedef 	TImageItem*      	      (*	REQUESTIMAGELIST)(int* count);
typedef 	BOOL                    (*	REQUESTIMAGEFROMLIST)(TIPAddress ipAddr, int index);
typedef 	int              	      (*	GETIMAGEWIDTH)();
typedef 	int              	      (*	GETIMAGEHEIGHT)();
typedef 	EScanResolution         (*	GETIMAGERESOLUTION)();
typedef 	PIXEL_FORMAT*     	   (*	GETIMAGEBUFFER)();
typedef 	BOOL                    (*	SHOWCALIBRATIONDIALOG)(BOOL visible);
typedef 	void              	   (*	SHOWSCANNERCONTROLDIALOG)(BOOL visible);
typedef 	EIPSize                 (*	GETIPSIZE)();
typedef 	int               	   (*	GETPROGRESS)();
typedef 	EScannerStatus          (*	GETSCANNERSTATUS)();
typedef 	ERFIDStatus             (*	GETRFIDREADERSTATUS)();
typedef 	TUID                    (*	GETUID)(ERFIDNotify device);
typedef 	EConnectionType         (*	GETSCANNERCONNECTIONTYPE)();
typedef	BOOL                    (*	CLEARIMAGES)();
typedef	char *	               (*	GETPATIENTID)(ERFIDNotify device);
typedef	BOOL                    (*	SETPATIENTID)(char* pid, int size);
typedef	BOOL                    (* SLEEPSCANNER)(BOOL dosleep);
typedef 	long                    (* GETFIRMWAREVERSION)();
typedef  BOOL                    (* CONNECT)(char * ip);
typedef  const char *            (* DISCONNECT)();
typedef  ErrorCodes              (* GETERRORCODE)();
typedef  const char *            (* GETERRORMESSAGE)(ErrorCodes errCode);
typedef  ELANConnectType         (* GETLANCONNTYPE)();
typedef  bool                    (* SETOPTION)(int opt, int flag, void * optval, int valsize);

// Deprecated
typedef 	ELockResult(*LOCKSCANNER)();
typedef 	void(*UNLOCKSCANNER)();
typedef 	void(*FREEIMAGEBUFFER)();
typedef 	void(*SETRFIDLISTENING)(BOOL listening);
typedef	BOOL(*GETRFIDLISTENING)();
typedef 	EScannerError(*GETSCANNERERROR)();
typedef	BOOL(*DORUN)();
typedef 	void(*SETRESOLUTION)(EScanResolution res);
typedef	void(*SETAUTOCONTRAST)(BOOL autoContrast);
typedef	int(*GETSCANNERPN)(char * buffer, int buffer_size);
typedef 	void(*FREEUID)(TUID uid);