
// DentalCRSDKDemoDlg.h : header file
//

#pragma once
#include "SDK.h"
#include "afxcmn.h"
#include "afxwin.h"

class CImageListDlg;
class CScannerListDlg;

class ImageInfo {
public:
   int width;
   int height;
   unsigned short* data; // image data
   CStringA patientId;

   ImageInfo(int w, int h) {
      ASSERT(0 < w && 0 < h);
      width = w; height = h;
      data = new unsigned short[w * h];
   }
   ~ImageInfo() {
      delete[] data;
   }
};

enum {
   INPROGRESS_NONE,
   INPROGRESS_SCANIMAGE,
   INPROGRESS_GETIMAGE
};


// CDentalCRSDKDemoDlg dialog
class CDentalCRSDKDemoDlg : public CDialogEx
{
// Construction
public:
   CDentalCRSDKDemoDlg(CWnd* pParent = NULL);	// standard constructor

// Dialog Data
#ifdef AFX_DESIGN_TIME
	enum { IDD = IDD_DENTALCRSDKDEMO_DIALOG };
#endif

   CImageListDlg*		m_pImageListDlg;
   CScannerListDlg*	m_pScannerListDlg;

   ImageInfo* SetImageBuf(int width, int height, unsigned short* src, const char* patId);
   bool IsConnectedState() const { return estConnected == m_ScannerState; }
   bool CheckConnected(CWnd* pWnd = nullptr);
   void DisplayErrorMessage(const CString& title);
   bool IsNetReady() const;
   void ResetTransferProgressUI();
   void SetInProgress(int inProgress) { m_inProgress = inProgress; }
   void SetPatientInfo();

protected:
	virtual void DoDataExchange(CDataExchange* pDX);	// DDX/DDV support

private:
   EScannerStatus m_ScannerState;
   ERFIDStatus m_RFIDStatus;
   ImageInfo* m_pImgInfo;
   int m_inProgress;

// Implementation
protected:
	HICON m_hIcon;
   CProgressCtrl m_progress;
   CButton m_cbAutoContrast;

	// Generated message map functions
	virtual BOOL OnInitDialog();
	afx_msg void OnPaint();
	afx_msg HCURSOR OnQueryDragIcon();

   void EnableButtons(BOOL bConnected);
   void ResetScanInfoUI();
   bool IsScanOrGetImageInProgress() const 
   { return INPROGRESS_SCANIMAGE == m_inProgress || INPROGRESS_GETIMAGE == m_inProgress; }

	DECLARE_MESSAGE_MAP()
public:
   void UpdateScannerStatus(int status);
   void UpdateRFIDStatus(int status);
   void UpdateRFIDNotify(int notify);
   
   afx_msg void OnBnClickedBtnCloseScanner();
	afx_msg void OnBnClickedBtnOpenScanner();
	afx_msg void OnBnClickedBtnCalibration();
	afx_msg void OnBnClickedBtnScannercontrol();
	afx_msg void OnClose();
	afx_msg void OnBnClickedBtnImageList();
	afx_msg void OnBnClickedBtnSleep();
   afx_msg void OnBnClickedBtnScannerlist();
   afx_msg LRESULT OnScannerNotifyMsg(WPARAM wParam, LPARAM lParam);
   afx_msg LRESULT OnScannerScannerListMsg(WPARAM wParam, LPARAM lParam);
   afx_msg LRESULT OnScannerStatusMsg(WPARAM wParam, LPARAM lParam);
   afx_msg void OnBnClickedBtnSetpatient();
   afx_msg void OnTimer(UINT_PTR nIDEvent);
   afx_msg HBRUSH OnCtlColor(CDC* pDC, CWnd* pWnd, UINT nCtlColor);
};
