// ImageListDlg.cpp : implementation file
//

#include "stdafx.h"
#include "DentalCRSDKDemo.h"
#include "ImageListDlg.h"
#include "afxdialogex.h"
#include "DentalCRSDKDemoDlg.h"
#include "SDKString.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif


static const int IMAGELIST_MAX = 100;

// CImageListDlg dialog

IMPLEMENT_DYNAMIC(CImageListDlg, CDialogEx)

CImageListDlg::CImageListDlg(CWnd* pParent /*=NULL*/)
	: CDialogEx(IDD_IMAGELIST_DIALOG, pParent)
{

}

CImageListDlg::~CImageListDlg()
{
}

void CImageListDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialogEx::DoDataExchange(pDX);
	DDX_Control(pDX, IDC_IMAGE_LIST, m_ctrlImageList);
}


BEGIN_MESSAGE_MAP(CImageListDlg, CDialogEx)
	ON_BN_CLICKED(IDC_CLEAR_IMAGES, &CImageListDlg::OnBnClickedClearImages)
	ON_BN_CLICKED(IDC_REQUEST_IMAGE, &CImageListDlg::OnBnClickedReqeustImage)
	ON_BN_CLICKED(IDC_UPDATE_LIST, &CImageListDlg::OnBnClickedUpdateList)
   ON_NOTIFY(LVN_ITEMCHANGED, IDC_IMAGE_LIST, &CImageListDlg::OnLvnItemchangedImageList)
END_MESSAGE_MAP()


CDentalCRSDKDemoDlg* CImageListDlg::MainForm()
{
   CWnd* wnd = AfxGetApp()->GetMainWnd();
   if (wnd && IsWindow(wnd->GetSafeHwnd()))
      return static_cast<CDentalCRSDKDemoDlg*>(wnd);
   return nullptr;
}

// CImageListDlg message handlers

void CImageListDlg::InitList()
{
   const int nColumns = 8;
   TCHAR* strHead[nColumns] = { _T("No"), _T("Scanner"), _T("Saved"),
      _T("Served"), _T("IP Size"), _T("Res"), _T("UID"), _T("patient ID") };
	RECT rt; 
   int nCol;

   m_ctrlImageList.GetClientRect(&rt);
   int width[nColumns] = { 40,100,130,130,50,50,100,100 };
	width[6] = (rt.right - (rt.left + 500))/2 + 20;
	width[7] = (rt.right - (rt.left + 500))/2 - 20;

   for (int i = 0; i < nColumns; ++i) {
		nCol = m_ctrlImageList.InsertColumn(i, strHead[i], LVCFMT_LEFT, width[i]);
	}	

	m_ctrlImageList.SetExtendedStyle(LVS_EX_GRIDLINES|LVS_EX_FULLROWSELECT);
}

void CImageListDlg::OnBnClickedClearImages()
{
   if (!MainForm()->CheckConnected(this)) return;

   if (MessageBox(_T("If you want to clear all images in the Scanner\nClick \'yes\'"),
      _T("Clear images"), MB_YESNO | MB_ICONWARNING) == IDYES) {
      if (ClearImages())
         OnBnClickedUpdateList();
   }
}

void CImageListDlg::OnBnClickedReqeustImage()
{
   if (!MainForm()->CheckConnected(this)) return;

   MainForm()->ResetTransferProgressUI();

	POSITION pos = m_ctrlImageList.GetFirstSelectedItemPosition();
	if (pos) {
		int nItem = m_ctrlImageList.GetNextSelectedItem(pos);
		CString strSaved = m_ctrlImageList.GetItemText(nItem,2); //saved
		CString strIPAddr = m_ctrlImageList.GetItemText(nItem,1);

      TIPAddress ipaddr;
      ipaddr.quad = inet_addr(CW2A(strIPAddr));
      if (strSaved.Compare(_T("-")) == 0) {
         MessageBox(_T("Invalid scanned image."), _T("Download Image"), MB_OK | MB_ICONWARNING);
         return;
      }

      if (RequestImageFromList(ipaddr, nItem % IMAGELIST_MAX))
         MainForm()->SetInProgress(INPROGRESS_GETIMAGE);
      else
         MainForm()->DisplayErrorMessage(_T("Request Image"));
	}
}

void CImageListDlg::OnBnClickedUpdateList()
{
   TCHAR no[16];
   CString buf;
   sockaddr_in addr;

   if (!MainForm()->CheckConnected(this)) return;

   int count = 0;
   TImageItem* items = RequestImageList(&count);

   m_ctrlImageList.DeleteAllItems();

   for (int i = 0; i < count; ++i) {
      LVITEM lvItem;
      lvItem.mask = LVIF_TEXT;
      lvItem.iItem = i;
      lvItem.iSubItem = 0;
      _stprintf_s(no, 16, _T("%d"), items[i].index +1);
      lvItem.pszText = no;
      m_ctrlImageList.InsertItem(&lvItem);

      if (items[i].uid == 0 || items[i].uid == 0xFFFFFFFFFFFFFFFFLL) {
         addr.sin_addr.s_addr = items[i].owner.quad;
         buf = CA2W(inet_ntoa(addr.sin_addr));
         m_ctrlImageList.SetItemText(i, 1, buf);
         m_ctrlImageList.SetItemText(i, 2, _T("-"));
         m_ctrlImageList.SetItemText(i, 3, _T("-"));
         m_ctrlImageList.SetItemText(i, 4, _T("None"));
         m_ctrlImageList.SetItemText(i, 5, _T("-"));
         m_ctrlImageList.SetItemText(i, 6, _T("-"));
         m_ctrlImageList.SetItemText(i, 7, _T("-"));
      }
      else {
         // IP address
         addr.sin_addr.s_addr = items[i].owner.quad;
         buf = CA2W(inet_ntoa(addr.sin_addr));
         m_ctrlImageList.SetItemText(i, 1, buf);
         // saved and served date time
         try {
            COleDateTime dt(items[i].timeSaved);
            buf.Format(_T("%04d-%02d-%02d %02d:%02d:%02d"),
               dt.GetYear(), dt.GetMonth(), dt.GetDay(),
               dt.GetHour(), dt.GetMinute(), dt.GetSecond());
            m_ctrlImageList.SetItemText(i, 2, buf);
            dt = items[i].timeServed;
            buf.Format(_T("%04d-%02d-%02d %02d:%02d:%02d"),
               dt.GetYear(), dt.GetMonth(), dt.GetDay(),
               dt.GetHour(), dt.GetMinute(), dt.GetSecond());
            m_ctrlImageList.SetItemText(i, 3, buf);
         }
         catch (...) {
            m_ctrlImageList.SetItemText(i, 2, _T(""));
            m_ctrlImageList.SetItemText(i, 3, _T(""));
         }
         // IP size
         m_ctrlImageList.SetItemText(i, 4, strIPSizeList[items[i].ipSize + 1]);
         // scan resolution
         m_ctrlImageList.SetItemText(i, 5, items[i].resolution ? _T("HD") : _T("SD"));
         // IP's ID
         buf.Format(_T("%I64X"), items[i].uid);
         m_ctrlImageList.SetItemText(i, 6, buf);
         // patient ID
         m_ctrlImageList.SetItemText(i, 7, CA2W(items[i].pid, CP_UTF8));
      }
   }

   EnableButtons();
}


void CImageListDlg::OnLvnItemchangedImageList(NMHDR * /*pNMHDR*/, LRESULT *pResult)
{
   //LPNMLISTVIEW pNMLV = reinterpret_cast<LPNMLISTVIEW>(pNMHDR);
   
   EnableButtons();

   *pResult = 0;
}

void CImageListDlg::EnableButtons()
{
   GetDlgItem(IDC_REQUEST_IMAGE)->EnableWindow(m_ctrlImageList.GetSelectedCount() == 1);
}
