// ScannerListDlg.cpp : implementation file
//

#include "stdafx.h"
#include "DentalCRSDKDemo.h"
#include "ScannerListDlg.h"
#include "afxdialogex.h"
#include "DentalCRSDKDemoDlg.h"
#include "SDKString.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif


// CScannerListDlg dialog

IMPLEMENT_DYNAMIC(CScannerListDlg, CDialogEx)

CScannerListDlg::CScannerListDlg(CWnd* pParent /*=NULL*/)
	: CDialogEx(IDD_SCANNER_LIST, pParent)
{

}

CScannerListDlg::~CScannerListDlg()
{
}

void CScannerListDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialogEx::DoDataExchange(pDX);
	DDX_Control(pDX, IDC_SCANNER_LIST, m_ctrlScannerList);
}


BEGIN_MESSAGE_MAP(CScannerListDlg, CDialogEx)
	ON_BN_CLICKED(IDC_UPDATE_LIST, &CScannerListDlg::OnBnClickedUpdateList)
   ON_BN_CLICKED(IDC_CONNECT_SCANNER, &CScannerListDlg::OnBnClickedConnectScanner)
   ON_BN_CLICKED(IDC_DISCONNECT_SCANNER, &CScannerListDlg::OnBnClickedDisconnectScanner)
   ON_NOTIFY(LVN_ITEMCHANGED, IDC_SCANNER_LIST, &CScannerListDlg::OnLvnItemchangedScannerList)
END_MESSAGE_MAP()

void CScannerListDlg::InitScannerList()
{
   const int nColumns = 5;
   TCHAR* strHead[nColumns] = { _T("Nickname"), _T("IP Address"), _T("Status"),
      _T("Image"), _T("Host Name") };
	int width[nColumns] = {150,100,100,50,150};
   RECT rt;
   int nCol;

   m_ctrlScannerList.GetClientRect(&rt);
   int total = rt.right - rt.left;
	int longcolumn =  (total - (width[0] + width[2] + width[3]))/2;
	width[1] = longcolumn;
	width[4] = longcolumn;

   for (int i = 0; i < nColumns; ++i) {
		nCol = m_ctrlScannerList.InsertColumn(i, strHead[i], LVCFMT_LEFT, width[i]);
	}	

	m_ctrlScannerList.SetExtendedStyle(LVS_EX_GRIDLINES|LVS_EX_FULLROWSELECT);
}

CDentalCRSDKDemoDlg* CScannerListDlg::MainForm()
{
   CWnd* wnd = AfxGetApp()->GetMainWnd();
   if (wnd && IsWindow(wnd->GetSafeHwnd()))
      return static_cast<CDentalCRSDKDemoDlg*>(wnd);
   return nullptr;
}


// CScannerListDlg message handlers

void CScannerListDlg::OnBnClickedUpdateList()
{
   TCHAR scannerName[20];
   CString buf;

   int count = 0;
   const TScannerItem* items = RequestScannerList(&count);

   m_ctrlScannerList.DeleteAllItems();

   for (int i = 0; i < count; i++) {
      // scanner name
      TIPAddress ip = items[i].scannerIP;
      LVITEM lvItem;
      lvItem.mask = LVIF_TEXT;
      lvItem.iItem = i;
      lvItem.iSubItem = 0;
      lstrcpy(scannerName, CA2W(items[i].scannerName));
      lvItem.pszText = scannerName;
      m_ctrlScannerList.InsertItem(&lvItem);
      
      // IP address
      m_ctrlScannerList.SetItemText(i, 1, CA2W(ip.toString().c_str()));
      // scanner status
      m_ctrlScannerList.SetItemText(i, 2, unitStatusMsg[(int)items[i].unitStatus]);
      // image count
      buf.Format(_T("%03d"), items[i].imageCount);
      m_ctrlScannerList.SetItemText(i, 3, buf);
      // host connected
      m_ctrlScannerList.SetItemText(i, 4, CA2W(items[i].hostName));
   }

   EnableButtons();
}

// connect manually
void CScannerListDlg::OnBnClickedConnectScanner()
{
   CString msg;

   if (MainForm()->IsNetReady()) {
      POSITION pos = m_ctrlScannerList.GetFirstSelectedItemPosition();
      if (pos) {
         int item = m_ctrlScannerList.GetNextSelectedItem(pos);
         CString ipAddr = m_ctrlScannerList.GetItemText(item, 1);
         CString scannarName = m_ctrlScannerList.GetItemText(item, 0);

         if (Connect(CW2A(ipAddr))) {
            msg.Format(_T("Connected to %s(%s)"), scannarName, ipAddr);
            MessageBox(msg, _T("Connect Scanner"), MB_OK | MB_ICONINFORMATION);
         }
         else
            MainForm()->DisplayErrorMessage(_T("Connect Scanner"));
      }
   }
   else
      MessageBox(_T("Invalid state to connect."), _T("Connect Scanner"), MB_OK | MB_ICONWARNING);
}

// disconnect manually
void CScannerListDlg::OnBnClickedDisconnectScanner()
{
   CString msg;

   const char* ipAddr = Disconnect();
   if (ipAddr) {
      msg.Format(_T("Disconnected from the scanner(%s)"), LPCTSTR(CA2W(ipAddr)));
      MessageBox(msg, _T("Disconnect Scanner"), MB_OK | MB_ICONINFORMATION);
   }
   else
      MainForm()->DisplayErrorMessage(_T("Disconnect Scanner"));
}

void CScannerListDlg::EnableButtons()
{
   int nSel = m_ctrlScannerList.GetSelectedCount();
   const bool isNet = (GetScannerConnectionType() == ectNtoN);

   GetDlgItem(IDC_CONNECT_SCANNER)->EnableWindow(1 == nSel && MainForm()->IsNetReady());
   GetDlgItem(IDC_DISCONNECT_SCANNER)->EnableWindow(isNet && MainForm()->IsConnectedState());
}

void CScannerListDlg::OnLvnItemchangedScannerList(NMHDR * /*pNMHDR*/, LRESULT *pResult)
{
   //LPNMLISTVIEW pNMLV = reinterpret_cast<LPNMLISTVIEW>(pNMHDR);
   
   EnableButtons();

   *pResult = 0;
}
